"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.watchPostGraphQLSchema = exports.createPostGraphQLSchema = exports.postGraphQLClassicIdsInflection = exports.postGraphQLInflection = exports.postGraphQLClassicIdsOverrides = exports.postGraphQLBaseOverrides = exports.inflections = undefined;

var _assign = require("babel-runtime/core-js/object/assign");

var _assign2 = _interopRequireDefault(_assign);

var _graphileBuild = require("graphile-build");

var _graphileBuildPg = require("graphile-build-pg");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const ensureValidPlugins = (name, arr) => {
  if (!Array.isArray(arr)) {
    throw new Error(`Option '${name}' should be an array`);
  }
  for (let i = 0, l = arr.length; i < l; i++) {
    const fn = arr[i];
    if (typeof fn !== "function") {
      throw new Error(`Option '${name}' should be an array of functions, found '${typeof fn}' at index ${i}`);
    }
  }
};

exports.inflections = _graphileBuildPg.inflections;
const postGraphQLBaseOverrides = exports.postGraphQLBaseOverrides = {
  enumName(value) {
    return _graphileBuildPg.inflections.defaultUtils.constantCase(value);
  }
};

const postGraphQLClassicIdsOverrides = exports.postGraphQLClassicIdsOverrides = {
  column(name, _table, _schema) {
    return name === "id" ? "rowId" : _graphileBuildPg.inflections.defaultUtils.camelCase(name);
  }
};

const postGraphQLInflection = exports.postGraphQLInflection = _graphileBuildPg.inflections.newInflector(postGraphQLBaseOverrides);
const postGraphQLClassicIdsInflection = exports.postGraphQLClassicIdsInflection = _graphileBuildPg.inflections.newInflector((0, _assign2.default)({}, postGraphQLBaseOverrides, postGraphQLClassicIdsOverrides));

const getPostGraphQLBuilder = async (pgConfig, schemas, options = {}) => {
  const { dynamicJson, classicIds, nodeIdFieldName } = options;
  const {
    replaceAllPlugins,
    appendPlugins = [],
    prependPlugins = [],
    jwtPgTypeIdentifier,
    jwtSecret,
    disableDefaultMutations,
    graphqlBuildOptions,
    inflector
  } = options;
  if (replaceAllPlugins) {
    ensureValidPlugins("replaceAllPlugins", replaceAllPlugins);
    if (prependPlugins && prependPlugins.length || appendPlugins && appendPlugins.length) {
      throw new Error("When using 'replaceAllPlugins' you must not specify 'appendPlugins'/'prependPlugins'");
    }
  }
  ensureValidPlugins("prependPlugins", prependPlugins);
  ensureValidPlugins("appendPlugins", appendPlugins);
  return (0, _graphileBuild.getBuilder)(replaceAllPlugins ? [...prependPlugins, ...replaceAllPlugins, ...appendPlugins] : [...prependPlugins, ..._graphileBuild.defaultPlugins, ..._graphileBuildPg.defaultPlugins, ...appendPlugins], (0, _assign2.default)({
    pgConfig: pgConfig,
    pgSchemas: Array.isArray(schemas) ? schemas : [schemas],
    pgExtendedTypes: !!dynamicJson,
    pgInflection: inflector || (classicIds ? postGraphQLClassicIdsInflection : postGraphQLInflection),
    nodeIdFieldName: nodeIdFieldName || (classicIds ? "id" : "nodeId"),
    pgJwtTypeIdentifier: jwtPgTypeIdentifier,
    pgJwtSecret: jwtSecret,
    pgDisableDefaultMutations: disableDefaultMutations
  }, graphqlBuildOptions));
};

const createPostGraphQLSchema = exports.createPostGraphQLSchema = async (pgConfig, schemas, options = {}) => {
  const builder = await getPostGraphQLBuilder(pgConfig, schemas, options);
  return builder.buildSchema();
};

/*
 * Unless an error occurs, `onNewSchema` is guaranteed to be called before this promise resolves
 */
const watchPostGraphQLSchema = exports.watchPostGraphQLSchema = async (pgConfig, schemas, options = {}, onNewSchema) => {
  if (typeof onNewSchema !== "function") {
    throw new Error("You cannot call watchPostGraphQLSchema without a function to pass new schemas to");
  }
  const builder = await getPostGraphQLBuilder(pgConfig, schemas, options);
  let released = false;
  await builder.watchSchema(onNewSchema);

  return async function release() {
    if (released) return;
    released = true;
    await builder.unwatchSchema();
  };
};
//# sourceMappingURL=index.js.map